/* Bounds Checking for GCC.
   Copyright (C) 1995 Richard W.M. Jones <rjones@orchestream.com>.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

   File:
  	lib/functions.h
   Summary:
  	Header file describing the function stack used by "functions.c".
   Other notes:
  	
   Author      	Date		Notes
   RWMJ		6/3/95		Initial implementation.
*/

#ifndef _FUNCTIONS_H_
#define _FUNCTIONS_H_

/* This is the structure used to store a record of a single function call.  */

typedef struct _function_context
{
  int nesting_nr;			/* nesting level nr.  */
  unsigned char in_main;		/* True if this is `main ()'.  */
  unsigned char no_checking;		/* True if we don't check this fn.  */
  const char *name;			/* Pointer to function name.  */
  const char *filename;			/* File where declared.  */
  int line;				/* Line where declared.  */
  object parms;				/* Parms for this function.  */
  object stack;				/* Stack elements for this function.  */
  alloca_item *alloca_stack;		/* List of calls to alloca.  */
  struct _function_context *next;	/* Next function_context.  */
  struct _function_context *prev;	/* Previous function_context.  */
} function_context;

typedef struct _cur_context
{
  int current_thread;
  int nesting_nr;
  /* The current function context.  */
  function_context func_context;
  function_context *hist_func;
  char *last_end;       /* strtok last_end pointer.  */
  struct _cur_context *next;
} cur_context;

extern char **__bounds_strtok_last_end (void);
extern void __bounds_add_function_object (object *obj, int nesting_nr);
extern int  __bounds_remove_function_object (object *obj, int nesting_nr);

#endif /* _FUNCTIONS_H_ */
