/* Bounds Checking for GCC.
   Copyright (C) 1995 Richard W.M. Jones <rjones@orchestream.com>.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

   File:
  	lib/malloc/mmap.c
   Summary:
  	Bounds checking interface to the GNU C malloc library. The GNU
  	malloc has been modified to provide the unchecked functions
  	like '__bounds_malloc' directly. We provide the checked versions
  	(eg. 'mmap') here.
   Other notes:
  	
   Author      	Date		Notes
   HtB		1/7/99		Initial implementation.
*/

#include <stddef.h>
#include <unistd.h>
#include <sys/types.h>
#include "../bounds-lib.h"
#include "../mutex.h"			/* for 'enter|leave_critical_section'*/

void *mmap (void  *start, size_t size, int prot,
	    int flags, int fd, off_t offset);
int munmap (void *start, size_t size);

void *
__bounds_check_mmap (const char *filename, int line,
		     void  *start,  size_t size, int prot,
		     int flags, int fd, off_t offset)
{
  void *result;
  sigset_t old_mask;

  enter_critical_section (&old_mask);
  result = mmap (start, size, prot, flags, fd, offset);
  if (result != (void *)-1)
    __bounds_add_heap_object (result, size, 1, "mmap", 1, filename, line);
  leave_critical_section (&old_mask);

  return result;
}

int
__bounds_check_munmap (const char *filename, int line, void *start, size_t size)
{
  int result;
  sigset_t old_mask;

  enter_critical_section (&old_mask);
  result = munmap (start, size);
  if (result == 0)
    __bounds_delete_heap_object (start, size,
				 "invalid pointer passed to `munmap'",
				 filename, line);
  leave_critical_section (&old_mask);

  return result;
}
